
#include <iostream>
#include <fstream>

#include <time.h>

#include "./headers/cu_funcs.h"
#include "./headers/adv.h"


const int mesh = 10000; // actual computational grid
const int r = 5; // ta3h akribeias ston a3ona x

const int threadsperblock = 64;

const int length = mesh + 2*r ;

const int tsteps = 38000000;

const int thalf = 16000000;

using namespace std;


int main (void) {

	float * ro, * rod, * yu, * yud, * en, *end; // device
	float * ronew, * rodnew, * yunew, * yudnew, * ennew, * endnew; //device
	float h_ro[length], h_yu[length]; //host


	cudaMalloc((void**)&ro, length*sizeof(float));
	cudaMalloc((void**)&rod, length*sizeof(float));
	cudaMalloc((void**)&yu, length*sizeof(float));
	cudaMalloc((void**)&yud, length*sizeof(float));
	cudaMalloc((void**)&en, length*sizeof(float));
	cudaMalloc((void**)&end, length*sizeof(float));
	
	//new-mems
	cudaMalloc((void**)&ronew, length*sizeof(float));
	cudaMalloc((void**)&rodnew, length*sizeof(float));
	cudaMalloc((void**)&yunew, length*sizeof(float));
	cudaMalloc((void**)&yudnew, length*sizeof(float));
	cudaMalloc((void**)&ennew, length*sizeof(float));
	cudaMalloc((void**)&endnew, length*sizeof(float));

	
	dim3 dimGrid(int(ceil(float(length)/float(threadsperblock))));
	dim3 dimBlock(threadsperblock);

	
	cout << "cuda_blast runs with:" << endl;
	cout << "blocks in grid (x-dimension):" << dimGrid.x << endl;
	cout << "threads per block (x-dimension):" << dimBlock.x << endl << endl;

	
	init_cond <<<dimGrid, dimBlock>>> (ro, rod, yu, yud, en, end, length,mesh,r);

	
	ofstream outputr, outputr_h;
	outputr.open("data_r.dat");
	outputr_h.open("data_rh.dat");
	
	
	
	int t,i;
		
	
	clock_t start = clock();
	
	
	for (t=1; t<=tsteps; t++){
	

		advance <<<dimGrid, dimBlock>>> (ro,rod,yu,yud,en,end,ronew,rodnew,yunew,yudnew,ennew,endnew,mesh,r);
		
		if (t == thalf) {
			
			cudaMemcpy (h_ro,ronew, length*sizeof(float), cudaMemcpyDeviceToHost);
	
		}
		
		refresh  <<<dimGrid, dimBlock>>> (ro,rod,yu,yud,en,end,ronew,rodnew,yunew,yudnew,ennew,endnew,length,mesh,r);
	
		if (t == thalf) {
			
			for (i=0; i< length ;i++) outputr_h << i << "\t" << h_ro[i] << endl ;
		
		}
		
	}

	
	cudaMemcpy (h_ro,ro, length*sizeof(float), cudaMemcpyDeviceToHost);
	cudaMemcpy (h_yu,yu, length*sizeof(float), cudaMemcpyDeviceToHost);

	cout << "elapsed time : " << ( (double)clock() - start) / CLOCKS_PER_SEC << "secs" << endl << endl;

				
	

	for (i=0; i< length ;i++) outputr << i << "\t" << h_ro[i] << endl ;
	
	
	outputr.close();
	outputr_h.close();
	
	cudaFree(ro);
	cudaFree(rod);
	cudaFree(yu);
	cudaFree(yud);
	cudaFree(en);
	cudaFree(end);
	cudaFree(ronew);
	cudaFree(rodnew);
	cudaFree(yunew);
	cudaFree(yudnew);
	cudaFree(ennew);
	cudaFree(endnew);
		
	
	
	
	return 0;



}

