
#include <iostream>
#include <fstream>


#include "./headers/cu_funcs.h"
#include "./headers/adv.h"


const int mesh = 800; // actual computational grid
const int r = 5; // ta3h akribeias ston a3ona x

const int threadsperblock = 64;

const int length = mesh + 2*r ;

const int tsteps = 380000;

using namespace std;


int main (void) {

	float * ro, * rod, * yu, * yud, * en, *end; // device
	float * ronew, * rodnew, * yunew, * yudnew, * ennew, * endnew; //device
	float h_ro[length], h_yu[length]; //host


	
	cudaMalloc((void**)&ro, length*sizeof(float));
	cudaMalloc((void**)&rod, length*sizeof(float));
	cudaMalloc((void**)&yu, length*sizeof(float));
	cudaMalloc((void**)&yud, length*sizeof(float));
	cudaMalloc((void**)&en, length*sizeof(float));
	cudaMalloc((void**)&end, length*sizeof(float));
	
	//new-mems
	cudaMalloc((void**)&ronew, length*sizeof(float));
	cudaMalloc((void**)&rodnew, length*sizeof(float));
	cudaMalloc((void**)&yunew, length*sizeof(float));
	cudaMalloc((void**)&yudnew, length*sizeof(float));
	cudaMalloc((void**)&ennew, length*sizeof(float));
	cudaMalloc((void**)&endnew, length*sizeof(float));

	
	dim3 dimGrid(int(ceil(float(length)/float(threadsperblock))));
	dim3 dimBlock(threadsperblock);


	cout << "cuda_blast_RK runs with:" << endl;
	cout << "blocks in grid (x-dimension):" << dimGrid.x << endl;
	cout << "threads per block (x-dimension):" << dimBlock.x << endl << endl;

	
	init_cond <<<dimGrid, dimBlock>>> (ro, rod, yu, yud, en, end, length,mesh,r);

		
	///////////////////
	//   timer code  //
	///////////////////
	cudaEvent_t start, stop;
	float time;
	cudaEventCreate(&start);
	cudaEventCreate(&stop);
	////////////////////
	////////////////////
	
	int t;
		
	cudaEventRecord( start, 0 );
	
	
	for (t=1; t<=tsteps; t++){
	

		advance <<<dimGrid, dimBlock>>> (ro,rod,yu,yud,en,end,ronew,rodnew,yunew,yudnew,ennew,endnew,mesh,r);
		
		refresh  <<<dimGrid, dimBlock>>> (ro,rod,yu,yud,en,end,ronew,rodnew,yunew,yudnew,ennew,endnew,length,mesh,r);
					
	}

	cudaEventRecord( stop, 0 );
	cudaEventSynchronize( stop );
	
	
	///////////////////
	//   timer code  //
	///////////////////
	cudaEventElapsedTime( &time, start, stop );
	cudaEventDestroy( start );
	cudaEventDestroy( stop );
	//////////////////
	//////////////////
	
	cout << time<< endl << endl;
	
	
	cudaMemcpy (h_ro,ro, length*sizeof(float), cudaMemcpyDeviceToHost);
	cudaMemcpy (h_yu,yu, length*sizeof(float), cudaMemcpyDeviceToHost);

	
	
	ofstream outputr;
	outputr.open("data_r.dat");
			
	int i;

	for (i=0; i< length ;i++) outputr << i << "\t" << h_ro[i] << endl ;
	
	outputr.close();

	
	cudaFree(ro);
	cudaFree(rod);
	cudaFree(yu);
	cudaFree(yud);
	cudaFree(en);
	cudaFree(end);
	cudaFree(ronew);
	cudaFree(rodnew);
	cudaFree(yunew);
	cudaFree(yudnew);
	cudaFree(ennew);
	cudaFree(endnew);
		
	
	
	
	return 0;



}

